@@ -10,7 +10,7 @@ if TYPE_CHECKING:
     T = TypeVar('T', bound=BaseMongoObject)
 
 
-class BaseIDQuery():
+class IDQuery():
 
     def __init__(self,
                  cls: type[T],
@@ -18,31 +18,23 @@ class BaseIDQuery():
         self.cls = cls
         self.id = ObjectId(id)
 
-    def _fetch(self) -> T:
+    def exec(self) -> T:
         fetched_object = self.cls.collection().find_one({'_id': self.id})
         if fetched_object is not None:
             return Decoder().decode_root(fetched_object, self.cls)
         raise ObjectNotFoundException(
             f'{self.cls.__name__}(_id={self.id}) not found.')
 
-
-class IDQuery(BaseIDQuery):
-
     def __await__(self) -> Generator[None, None, T]:
         yield
-        return self._fetch()
-
-    @property
-    def optional(self) -> OptionalIDQuery:
-        return OptionalIDQuery(cls=self.cls, id=self.id)
+        return self.exec()
 
 
-class OptionalIDQuery(BaseIDQuery):
+class OptionalIDQuery(IDQuery):
 
-    def __await__(self) -> Generator[None, None, Optional[T]]:
-        yield
+    def exec(self) -> Optional[T]:
         try:
-            return self._fetch()
+            return super().exec()
         except ObjectNotFoundException:
             return None
 
@@ -57,7 +49,7 @@ class BaseQuery():
         self.skipping: Optional[int] = None
         self.limiting: Optional[int] = None
 
-    def _fetch(self) -> list[T]:
+    def exec(self) -> list[T]:
         kwargs: dict[str, Any] = {}
         if self.filter is not None:
             kwargs['filter'] = self.filter
@@ -74,6 +66,10 @@ class BaseQuery():
         coder = Decoder()
         return list(map(lambda doc: coder.decode_root(doc, self.cls), results))
 
+    def __await__(self) -> Generator[None, None, list[T]]:
+        yield
+        return self.exec()
+
 
 class ListQuery(BaseQuery):
 
@@ -87,10 +83,6 @@ class ListQuery(BaseQuery):
         self.filter = filter
         return self
 
-    def __await__(self) -> Generator[None, None, list[T]]:
-        yield
-        return self._fetch()
-
     def where(self, filter: dict[str, Any]) -> ListQuery:
         self.filter = filter
         return self
@@ -134,9 +126,13 @@ class ListQuery(BaseQuery):
         return self
 
     @property
-    def one(self) -> SingleQuery:
+    def first(self) -> SingleQuery:
         return SingleQuery(self)
 
+    @property
+    def first_or_none(self) -> OptionalSingleQuery:
+        return OptionalSingleQuery(SingleQuery(self))
+
 
 class SingleQuery(BaseQuery):
 
@@ -148,9 +144,8 @@ class SingleQuery(BaseQuery):
         self.skipping = query.skipping
         self.limiting = 1
 
-    def __await__(self) -> Generator[None, None, T]:
-        yield
-        results: list[T] = self._fetch()
+    def exec(self) -> T:
+        results: list[T] = super().exec()
         if len(results) < 1:
             raise ObjectNotFoundException(
                 f'{self.cls.__name__}(filter={self.filter}, '
@@ -158,10 +153,6 @@ class SingleQuery(BaseQuery):
                 f'skipping={self.skipping}) not found.')
         return results[0]
 
-    @property
-    def optional(self) -> OptionalSingleQuery:
-        return OptionalSingleQuery(self)
-
 
 class OptionalSingleQuery(BaseQuery):
 
@@ -173,9 +164,8 @@ class OptionalSingleQuery(BaseQuery):
         self.skipping = query.skipping
         self.limiting = 1
 
-    def __await__(self) -> Generator[None, None, Optional[T]]:
-        yield
-        results: list[T] = self._fetch()
+    def exec(self) -> Optional[T]:
+        results: list[T] = super().exec()
         if len(results) < 1:
             return None
         return results[0]
