@@ -1,15 +1,15 @@
 from __future__ import annotations
-import unittest
-from typing import List, Dict
-from dotenv import load_dotenv
+from unittest import TestCase
+from typing import List, Dict, cast
 from datetime import date, datetime
 from bson import ObjectId
 from jsonclasses import jsonclass, types
 from jsonclasses_pymongo import MongoObject
 from jsonclasses_pymongo.encoder import Encoder
+from jsonclasses_pymongo.command import InsertOneCommand
 
 
-class TestEncoder(unittest.TestCase):
+class TestEncoder(TestCase):
 
     def test_encode_str_into_str(self):
         @jsonclass
@@ -17,9 +17,11 @@ class TestEncoder(unittest.TestCase):
             val1: str
             val2: str
         simple_object = SimpleEncodeStr(val1='q', val2='e')
-        serialized = Encoder().encode_root(simple_object)[0].object
+        batch_command = Encoder().encode_root(simple_object)
+        insert_command = cast(InsertOneCommand, batch_command.commands[0])
+        serialized = insert_command.object
         self.assertEqual(set(serialized.keys()), set(
-            ['_id', 'createdAt', 'updatedAt', 'val1', 'val2']))
+            ['_id', 'createdAt', 'updatedAt', 'val1', 'val2', 'deletedAt']))
         self.assertIsInstance(serialized['_id'], ObjectId)
         self.assertEqual(serialized['val1'], 'q')
         self.assertEqual(serialized['val2'], 'e')
@@ -30,9 +32,11 @@ class TestEncoder(unittest.TestCase):
             age: int
             length: int
         simple_object = SimpleEncodeInt(age=4, length=8)
-        serialized = Encoder().encode_root(simple_object)[0].object
+        batch_command = Encoder().encode_root(simple_object)
+        insert_command = cast(InsertOneCommand, batch_command.commands[0])
+        serialized = insert_command.object
         self.assertEqual(set(serialized.keys()), set(
-            ['_id', 'createdAt', 'updatedAt', 'age', 'length']))
+            ['_id', 'createdAt', 'updatedAt', 'age', 'length', 'deletedAt']))
         self.assertIsInstance(serialized['_id'], ObjectId)
         self.assertEqual(serialized['age'], 4)
         self.assertEqual(serialized['length'], 8)
@@ -43,9 +47,11 @@ class TestEncoder(unittest.TestCase):
             width: float
             length: float
         simple_object = SimpleEncodeFloat(width=4.5, length=8.5)
-        serialized = Encoder().encode_root(simple_object)[0].object
+        batch_command = Encoder().encode_root(simple_object)
+        insert_command = cast(InsertOneCommand, batch_command.commands[0])
+        serialized = insert_command.object
         self.assertEqual(set(serialized.keys()), set(
-            ['_id', 'createdAt', 'updatedAt', 'width', 'length']))
+            ['_id', 'createdAt', 'updatedAt', 'width', 'length', 'deletedAt']))
         self.assertIsInstance(serialized['_id'], ObjectId)
         self.assertEqual(serialized['width'], 4.5)
         self.assertEqual(serialized['length'], 8.5)
@@ -56,9 +62,11 @@ class TestEncoder(unittest.TestCase):
             b1: bool
             b2: bool
         simple_object = SimpleEncodeBool(b1=True, b2=False)
-        serialized = Encoder().encode_root(simple_object)[0].object
+        batch_command = Encoder().encode_root(simple_object)
+        insert_command = cast(InsertOneCommand, batch_command.commands[0])
+        serialized = insert_command.object
         self.assertEqual(set(serialized.keys()), set(
-            ['_id', 'createdAt', 'updatedAt', 'b1', 'b2']))
+            ['_id', 'createdAt', 'updatedAt', 'b1', 'b2', 'deletedAt']))
         self.assertIsInstance(serialized['_id'], ObjectId)
         self.assertEqual(serialized['b1'], True)
         self.assertEqual(serialized['b2'], False)
@@ -69,9 +77,11 @@ class TestEncoder(unittest.TestCase):
             d1: date = date(2012, 9, 15)
             d2: date = date(2020, 9, 14)
         simple_object = SimpleEncodeDate()
-        serialized = Encoder().encode_root(simple_object)[0].object
+        batch_command = Encoder().encode_root(simple_object)
+        insert_command = cast(InsertOneCommand, batch_command.commands[0])
+        serialized = insert_command.object
         self.assertEqual(set(serialized.keys()), set(
-            ['_id', 'createdAt', 'updatedAt', 'd1', 'd2']))
+            ['_id', 'createdAt', 'updatedAt', 'd1', 'd2', 'deletedAt']))
         self.assertIsInstance(serialized['_id'], ObjectId)
         self.assertEqual(serialized['d1'], datetime(2012, 9, 15, 0, 0, 0))
         self.assertEqual(serialized['d2'], datetime(2020, 9, 14, 0, 0, 0))
@@ -82,9 +92,11 @@ class TestEncoder(unittest.TestCase):
             d1: date = datetime(2012, 9, 15, 0, 0, 0)
             d2: date = datetime(2020, 9, 14, 0, 0, 0)
         simple_object = SimpleEncodeDatetime()
-        serialized = Encoder().encode_root(simple_object)[0].object
+        batch_command = Encoder().encode_root(simple_object)
+        insert_command = cast(InsertOneCommand, batch_command.commands[0])
+        serialized = insert_command.object
         self.assertEqual(set(serialized.keys()), set(
-            ['_id', 'createdAt', 'updatedAt', 'd1', 'd2']))
+            ['_id', 'createdAt', 'updatedAt', 'd1', 'd2', 'deletedAt']))
         self.assertIsInstance(serialized['_id'], ObjectId)
         self.assertEqual(serialized['d1'], datetime(2012, 9, 15, 0, 0, 0))
         self.assertEqual(serialized['d2'], datetime(2020, 9, 14, 0, 0, 0))
@@ -95,11 +107,15 @@ class TestEncoder(unittest.TestCase):
             str_values: List[str]
             int_values: List[int]
             bool_values: List[bool]
-        simple_object = SimpleEncodeScalarTypesList(str_values=['0', '1'], int_values=[
-                                                    0, 1], bool_values=[False, True])
-        serialized = Encoder().encode_root(simple_object)[0].object
+        simple_object = SimpleEncodeScalarTypesList(str_values=['0', '1'],
+                                                    int_values=[0, 1],
+                                                    bool_values=[False, True])
+        batch_command = Encoder().encode_root(simple_object)
+        insert_command = cast(InsertOneCommand, batch_command.commands[0])
+        serialized = insert_command.object
         self.assertEqual(set(serialized.keys()), set(
-            ['_id', 'createdAt', 'updatedAt', 'strValues', 'intValues', 'boolValues']))
+            ['_id', 'createdAt', 'updatedAt', 'strValues', 'intValues',
+             'boolValues', 'deletedAt']))
         self.assertIsInstance(serialized['_id'], ObjectId)
         self.assertEqual(serialized['strValues'], ['0', '1'])
         self.assertEqual(serialized['intValues'], [0, 1])
@@ -111,9 +127,11 @@ class TestEncoder(unittest.TestCase):
             str_values: Dict[str, str]
         simple_object = SimpleEncodeScalarTypesDict(
             str_values={'0': 'zero', '1': 'one'})
-        serialized = Encoder().encode_root(simple_object)[0].object
+        batch_command = Encoder().encode_root(simple_object)
+        insert_command = cast(InsertOneCommand, batch_command.commands[0])
+        serialized = insert_command.object
         self.assertEqual(set(serialized.keys()), set(
-            ['_id', 'createdAt', 'updatedAt', 'strValues']))
+            ['_id', 'createdAt', 'updatedAt', 'strValues', 'deletedAt']))
         self.assertIsInstance(serialized['_id'], ObjectId)
         self.assertEqual(serialized['strValues'], {'0': 'zero', '1': 'one'})
 
@@ -126,9 +144,11 @@ class TestEncoder(unittest.TestCase):
             })
         simple_object = SimpleEncodeScalarTypesShape(
             vals={'0': 'zero', '1': 1})
-        serialized = Encoder().encode_root(simple_object)[0].object
+        batch_command = Encoder().encode_root(simple_object)
+        insert_command = cast(InsertOneCommand, batch_command.commands[0])
+        serialized = insert_command.object
         self.assertEqual(set(serialized.keys()), set(
-            ['_id', 'createdAt', 'updatedAt', 'vals']))
+            ['_id', 'createdAt', 'updatedAt', 'vals', 'deletedAt']))
         self.assertIsInstance(serialized['_id'], ObjectId)
         self.assertEqual(serialized['vals'], {'0': 'zero', '1': 1})
 
@@ -142,7 +162,8 @@ class TestEncoder(unittest.TestCase):
             address: SimpleEncodeEmbeddedInstanceAddress
         simple_object = SimpleEncodeEmbeddedInstance(
             address={'line1': 'Flam Road'})
-        commands = Encoder().encode_root(simple_object)
+        batch_command = Encoder().encode_root(simple_object)
+        commands = batch_command.commands
         self.assertEqual(len(commands), 1)
         command = commands[0]
         self.assertIs(command.collection,
@@ -165,7 +186,8 @@ class TestEncoder(unittest.TestCase):
                 SimpleEncodeForeignKeyInstanceAddress).linkedby('owner')
         simple_object = SimpleEncodeForeignKeyInstance(
             address={'line1': 'Flam Road'})
-        commands = Encoder().encode_root(simple_object)
+        batch_command = Encoder().encode_root(simple_object)
+        commands = batch_command.commands
         self.assertEqual(len(commands), 2)
         address_data = commands[0].object
         owner_data = commands[1].object
@@ -180,11 +202,11 @@ class TestEncoder(unittest.TestCase):
 
         @jsonclass
         class SimpleEncodeLocalKeyInstance(MongoObject):
-            address: SimpleEncodeLocalKeyInstanceAddress = types.linkto.instanceof(
-                SimpleEncodeLocalKeyInstanceAddress)
+            address: SimpleEncodeLocalKeyInstanceAddress = types.linkto.instanceof(SimpleEncodeLocalKeyInstanceAddress)
         simple_object = SimpleEncodeLocalKeyInstance(
             address={'line1': 'Flam Road'})
-        commands = Encoder().encode_root(simple_object)
+        batch_command = Encoder().encode_root(simple_object)
+        commands = batch_command.commands
         self.assertEqual(len(commands), 2)
         address_data = commands[0].object
         owner_data = commands[1].object
@@ -200,7 +222,8 @@ class TestEncoder(unittest.TestCase):
             addresses: List[MediumEncodeEmbeddedInstanceAddress]
         medium_object = MediumEncodeEmbeddedInstance(
             addresses=[{'line1': 'Flam Road'}, {'line1': 'Plam Road'}])
-        commands = Encoder().encode_root(medium_object)
+        batch_command = Encoder().encode_root(medium_object)
+        commands = batch_command.commands
         self.assertEqual(len(commands), 1)
         command = commands[0]
         self.assertIs(command.collection,
@@ -216,7 +239,7 @@ class TestEncoder(unittest.TestCase):
         class MediumEncodeForeignKeyInstanceAddress(MongoObject):
             line1: str
             owner: MediumEncodeForeignKeyInstance = types.linkto.instanceof(
-                'SimpleEncodeForeignKeyInstance')
+                'MediumEncodeForeignKeyInstance')
 
         @jsonclass
         class MediumEncodeForeignKeyInstance(MongoObject):
@@ -224,7 +247,8 @@ class TestEncoder(unittest.TestCase):
                 MediumEncodeForeignKeyInstanceAddress).linkedby('owner')
         simple_object = MediumEncodeForeignKeyInstance(
             addresses=[{'line1': 'Flam Road'}, {'line1': 'Klam Road'}])
-        commands = Encoder().encode_root(simple_object)
+        batch_command = Encoder().encode_root(simple_object)
+        commands = batch_command.commands
         self.assertEqual(len(commands), 3)
         address_0_data = commands[0].object
         address_1_data = commands[1].object
@@ -245,7 +269,8 @@ class TestEncoder(unittest.TestCase):
                 MediumEncodeLocalKeyInstanceAddress).linkto
         simple_object = MediumEncodeLocalKeyInstance(
             addresses=[{'line1': 'Flam Road'}, {'line1': 'Klam Road'}])
-        commands = Encoder().encode_root(simple_object)
+        batch_command = Encoder().encode_root(simple_object)
+        commands = batch_command.commands
         self.assertEqual(len(commands), 3)
         address_0_data = commands[0].object
         address_1_data = commands[1].object
@@ -259,7 +284,8 @@ class TestEncoder(unittest.TestCase):
             val: Dict[str, str] = types.dictof(types.str)
         simple_object = MediumEncodeCamelizeDictKeys(
             val={'key_one': 'val_one', 'key_two': 'val_two'})
-        commands = Encoder().encode_root(simple_object)
+        batch_command = Encoder().encode_root(simple_object)
+        commands = batch_command.commands
         encoded_val = commands[0].object['val']
         self.assertEqual(
             encoded_val, {'keyOne': 'val_one', 'keyTwo': 'val_two'})
@@ -270,7 +296,8 @@ class TestEncoder(unittest.TestCase):
             val: Dict[str, str] = types.dictof(types.str)
         simple_object = MediumEncodeDoNotCamelizeDictKeys(
             val={'key_one': 'val_one', 'key_two': 'val_two'})
-        commands = Encoder().encode_root(simple_object)
+        batch_command = Encoder().encode_root(simple_object)
+        commands = batch_command.commands
         encoded_val = commands[0].object['val']
         self.assertEqual(
             encoded_val, {'key_one': 'val_one', 'key_two': 'val_two'})
@@ -284,7 +311,8 @@ class TestEncoder(unittest.TestCase):
             })
         simple_object = MediumEncodeCamelizeShapeKeys(
             val={'key_one': 'val_one', 'key_two': 'val_two'})
-        commands = Encoder().encode_root(simple_object)
+        batch_command = Encoder().encode_root(simple_object)
+        commands = batch_command.commands
         encoded_val = commands[0].object['val']
         self.assertEqual(
             encoded_val, {'keyOne': 'val_one', 'keyTwo': 'val_two'})
@@ -298,7 +326,8 @@ class TestEncoder(unittest.TestCase):
             })
         simple_object = MediumEncodeDoNotCamelizeShapeKeys(
             val={'key_one': 'val_one', 'key_two': 'val_two'})
-        commands = Encoder().encode_root(simple_object)
+        batch_command = Encoder().encode_root(simple_object)
+        commands = batch_command.commands
         encoded_val = commands[0].object['val']
         self.assertEqual(
             encoded_val, {'key_one': 'val_one', 'key_two': 'val_two'})
@@ -317,20 +346,6 @@ class TestEncoder(unittest.TestCase):
                 'DifficultEncodeLinkedThruA').linkedthru('blinks')
         instance_a = DifficultEncodeLinkedThruA(
             aval='A', blinks=[{'bval': 'B1'}, {'bval': 'B2'}])
-        commands = Encoder().encode_root(instance_a)
-        self.assertEqual(len(commands), 5)
-        self.assertEqual(
-            commands[0].object['difficultEncodeLinkedThruAId'], ObjectId(instance_a.id))
-        self.assertEqual(commands[0].object['difficultEncodeLinkedThruBId'], ObjectId(
-            instance_a.blinks[0].id))
-        self.assertEqual(commands[0].collection.name,
-                         'difficultencodelinkedthruasblinksdifficultencodelinkedthrubsalinks')
-        self.assertEqual(
-            commands[1].object['difficultEncodeLinkedThruAId'], ObjectId(instance_a.id))
-        self.assertEqual(commands[1].object['difficultEncodeLinkedThruBId'], ObjectId(
-            instance_a.blinks[1].id))
-        self.assertEqual(commands[1].collection.name,
-                         'difficultencodelinkedthruasblinksdifficultencodelinkedthrubsalinks')
-        self.assertEqual(commands[2].object['bval'], 'B1')
-        self.assertEqual(commands[3].object['bval'], 'B2')
-        self.assertEqual(commands[4].object['aval'], 'A')
+        batch_command = Encoder().encode_root(instance_a)
+        commands = batch_command.commands
+        self.assertEqual(len(commands), 7)
